package net.knarcraft.paidsigns.command;

import net.knarcraft.paidsigns.PaidSigns;
import net.knarcraft.paidsigns.container.PaidSign;
import net.knarcraft.paidsigns.container.PaidSignCondition;
import net.knarcraft.paidsigns.formatting.StringFormatter;
import net.knarcraft.paidsigns.formatting.TranslatableMessage;
import net.knarcraft.paidsigns.formatting.Translator;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.jetbrains.annotations.NotNull;

import java.util.Map;

import static net.knarcraft.paidsigns.formatting.StringFormatter.getTranslatedErrorMessage;
import static net.knarcraft.paidsigns.formatting.StringFormatter.replacePlaceholder;
import static net.knarcraft.paidsigns.formatting.StringFormatter.replacePlaceholders;
import static net.knarcraft.paidsigns.formatting.StringFormatter.translateBoolean;

/**
 * A representation of the command for listing information about paid signs
 */
public class ListCommand extends TokenizedCommand {

    @Override
    public boolean onCommand(@NotNull CommandSender sender, @NotNull Command command, @NotNull String label,
                             @NotNull String[] args) {
        super.onCommand(sender, command, label, args);
        if (argumentSize < 1) {
            displaySigns(sender);
            return true;
        } else if (argumentSize < 3) {
            return parsePaidSignSelection(sender);
        }
        return false;
    }

    /**
     * Displays all available signs
     *
     * @param sender <p>The command sender to display the signs to</p>
     */
    private void displaySigns(CommandSender sender) {
        StringBuilder signs = new StringBuilder();
        for (String signName : PaidSigns.getInstance().getSignManager().getAllPaidSigns().keySet()) {
            signs.append(StringFormatter.replacePlaceholder(Translator.getTranslatedMessage(
                    TranslatableMessage.PAID_SIGNS_INFO_FORMAT), "{name}", signName));
        }
        sender.sendMessage(StringFormatter.replacePlaceholder(Translator.getTranslatedMessage(
                TranslatableMessage.PAID_SIGNS_INFO), "{signs}", signs.toString()));
    }

    /**
     * Parses the given input and displays the wanted paid-sign information
     *
     * @param sender <p>The command sender to display the information to</p>
     * @return <p>True if successful. False if the input contained errors</p>
     */
    private boolean parsePaidSignSelection(CommandSender sender) {
        PaidSign paidSign = PaidSigns.getInstance().getSignManager().getPaidSign(arguments.get(0));
        if (paidSign == null) {
            sender.sendMessage(getTranslatedErrorMessage(TranslatableMessage.ERROR_PAID_SIGN_NOT_FOUND));
            return false;
        }
        if (argumentSize < 2) {
            displayPaidSign(sender, paidSign);
        } else {
            short lineNumber;
            try {
                lineNumber = (short) (Short.parseShort(arguments.get(1)) - 1);
                if (lineNumber < 0 || lineNumber > 3) {
                    sender.sendMessage(StringFormatter.getTranslatedErrorMessage(TranslatableMessage.ERROR_INVALID_NUMBER));
                    return false;
                }
            } catch (NumberFormatException exception) {
                sender.sendMessage(getTranslatedErrorMessage(TranslatableMessage.ERROR_INVALID_NUMBER));
                return false;
            }
            if (!paidSign.getConditions().containsKey(lineNumber)) {
                sender.sendMessage(replacePlaceholder(getTranslatedErrorMessage(
                        TranslatableMessage.ERROR_NO_SUCH_CONDITION), "{line}", String.valueOf(lineNumber)));
                return false;
            }
            PaidSignCondition condition = paidSign.getConditions().get(lineNumber);
            displayPaidSignCondition(sender, paidSign.getName(), lineNumber, condition);
        }
        return true;
    }

    /**
     * Displays information about a paid sign condition
     *
     * @param sender    <p>The command sender to display the information to</p>
     * @param signName  <p>The name of the sign to display the condition for</p>
     * @param signLine  <p>The line the condition is for</p>
     * @param condition <p>The condition to display information about</p>
     */
    private void displayPaidSignCondition(CommandSender sender, String signName, short signLine,
                                          PaidSignCondition condition) {
        sender.sendMessage(StringFormatter.replacePlaceholders(Translator.getTranslatedMessage(
                TranslatableMessage.PAID_SIGN_CONDITION_INFO), new String[]{"{name}", "{line}", "{match}", "{regex}",
                "{case}", "{color}"}, new String[]{signName, String.valueOf(signLine + 1), condition.getStringToMatch(),
                translateBoolean(condition.executeRegex()), translateBoolean(condition.ignoreCase()),
                translateBoolean(condition.ignoreColor())}));
    }

    /**
     * Displays information about a paid sign
     *
     * @param sender   <p>The command sender to display the information to</p>
     * @param paidSign <p>The paid sign to display information about</p>
     */
    private void displayPaidSign(CommandSender sender, PaidSign paidSign) {
        Map<Short, PaidSignCondition> signConditions = paidSign.getConditions();
        StringBuilder conditions = new StringBuilder();
        for (short lineIndex : signConditions.keySet()) {
            String format = Translator.getTranslatedMessage(TranslatableMessage.PAID_SIGN_INFO_CONDITION_FORMAT);
            conditions.append(StringFormatter.replacePlaceholders(format, new String[]{"{line}", "{condition}"},
                    new String[]{String.valueOf((lineIndex + 1)), signConditions.get(lineIndex).getStringToMatch()}));
        }

        sender.sendMessage(replacePlaceholders(Translator.getTranslatedMessage(
                TranslatableMessage.PAID_SIGN_INFO), new String[]{"{name}", "{cost}", "{permission}", "{case}",
                "{color}", "{any}", "{conditions}"}, new String[]{paidSign.getName(), String.valueOf(paidSign.getCost()),
                paidSign.getPermission(), translateBoolean(paidSign.getIgnoreCase()),
                translateBoolean(paidSign.getIgnoreColor()), translateBoolean(paidSign.matchAnyCondition()),
                conditions.toString()}));
    }

}
